INTERPRETER = """
<jdk version="2">
    <name value="Python ({project_name})" />
    <type value="Python SDK" />
    <homePath value="{project_path}\\venv\Scripts\python.exe" />
    <roots>
        <classPath>
            <root type="composite">
                <root url="file://$USER_HOME$/PycharmProjects/{project_name}/venv" type="simple" />
                <root url="file://$USER_HOME$/PycharmProjects/{project_name}/venv/Lib/site-packages" type="simple" />
            </root>
        </classPath>
        <sourcePath>
            <root type="composite" />
        </sourcePath>
    </roots>
    <additional ASSOCIATED_PROJECT_PATH="$USER_HOME$/PycharmProjects/{project_name}" />
</jdk>
"""

IML_FILE = """
<?xml version="1.0" encoding="UTF-8"?>
<module type="PYTHON_MODULE" version="4">
  <component name="NewModuleRootManager">
    <content url="file://$MODULE_DIR$">
      <sourceFolder url="file://$MODULE_DIR$/{source_dir}" isTestSource="false" />{unit_tests}{integration_tests}
      <excludeFolder url="file://$MODULE_DIR$/.idea" />
      <excludeFolder url="file://$MODULE_DIR$/.pybuilder" />
      <excludeFolder url="file://$MODULE_DIR$/target/dist" />
      <excludeFolder url="file://$MODULE_DIR$/venv" />
    </content>
    <orderEntry type="jdk" jdkName="{interpreter_name}" jdkType="Python SDK" />
    <orderEntry type="sourceFolder" forTests="false" />
  </component>
  <component name="PyDocumentationSettings">
    <option name="myDocStringFormat" value="Plain" />
  </component>
  <component name="TestRunnerService">
    <option name="projectConfiguration" value="Unittests" />
    <option name="PROJECT_TEST_RUNNER" value="Unittests" />
  </component>
</module>"""
IML_SOURCEFOLDER_TEMPLATE = """
<sourceFolder url="file://$MODULE_DIR$/{directory}" isTestSource="true" />"""

MODULES_FILE = """
<?xml version="1.0" encoding="UTF-8"?>
<project version="4">
  <component name="ProjectModuleManager">
    <modules>
      <module fileurl="file://$PROJECT_DIR$/.idea/{project_file_name}" filepath="$PROJECT_DIR$/.idea/{project_file_name}" />
    </modules>
  </component>
</project>"""

MISC_FILE = """
<?xml version="1.0" encoding="UTF-8"?>
<project version="4">
  <component name="JavaScriptSettings">
    <option name="languageLevel" value="ES6" />
  </component>
  <component name="ProjectRootManager" version="2" project-jdk-name="{project_interpreter_name}" project-jdk-type="Python SDK" />
</project>"""

WORKSPACE_FILE = """
<?xml version="1.0" encoding="UTF-8"?>
<project version="4">
  <component name="PropertiesComponent">
    <property name="RunOnceActivity.ShowReadmeOnStart" value="true" />
    <property name="WebServerToolWindowFactoryState" value="false" />
    <property name="last_opened_file_path" value="$PROJECT_DIR$/build.py" />
    <property name="settings.editor.selected.configurable" value="com.jetbrains.python.configuration.PyActiveSdkModuleConfigurable" />
  </component>
  <component name="RunManager" selected="Python.run tests">
    <configuration name="build (develop no-tests)" type="PythonConfigurationType" factoryName="Python">
      <module name="{project_name}" />
      <option name="INTERPRETER_OPTIONS" value="" />
      <option name="PARENT_ENVS" value="true" />
      <envs>
        <env name="PYTHONUNBUFFERED" value="1" />
      </envs>
      <option name="SDK_HOME" value="" />
      <option name="WORKING_DIRECTORY" value="$PROJECT_DIR$" />
      <option name="IS_MODULE_SDK" value="true" />
      <option name="ADD_CONTENT_ROOTS" value="true" />
      <option name="ADD_SOURCE_ROOTS" value="true" />
      <EXTENSION ID="PythonCoverageRunConfigurationExtension" runner="coverage.py" />
      <option name="SCRIPT_NAME" value="$PROJECT_DIR$/build.py" />
      <option name="PARAMETERS" value="pycharm_builder publish --environment=develop --exclude=run_unit_tests --exclude=run_integration_tests" />
      <option name="SHOW_COMMAND_LINE" value="false" />
      <option name="EMULATE_TERMINAL" value="false" />
      <option name="MODULE_MODE" value="false" />
      <option name="REDIRECT_INPUT" value="false" />
      <option name="INPUT_FILE" value="" />
      <method v="2" />
    </configuration>
    <configuration name="build (develop)" type="PythonConfigurationType" factoryName="Python">
      <module name="{project_name}" />
      <option name="INTERPRETER_OPTIONS" value="" />
      <option name="PARENT_ENVS" value="true" />
      <envs>
        <env name="PYTHONUNBUFFERED" value="1" />
      </envs>
      <option name="SDK_HOME" value="" />
      <option name="WORKING_DIRECTORY" value="$PROJECT_DIR$" />
      <option name="IS_MODULE_SDK" value="true" />
      <option name="ADD_CONTENT_ROOTS" value="true" />
      <option name="ADD_SOURCE_ROOTS" value="true" />
      <EXTENSION ID="PythonCoverageRunConfigurationExtension" runner="coverage.py" />
      <option name="SCRIPT_NAME" value="$PROJECT_DIR$/build.py" />
      <option name="PARAMETERS" value="pycharm_builder publish --environment=develop" />
      <option name="SHOW_COMMAND_LINE" value="false" />
      <option name="EMULATE_TERMINAL" value="false" />
      <option name="MODULE_MODE" value="false" />
      <option name="REDIRECT_INPUT" value="false" />
      <option name="INPUT_FILE" value="" />
      <method v="2" />
    </configuration>
    <configuration name="run tests (PyBuilder)" type="PythonConfigurationType" factoryName="Python">
      <module name="{project_name}" />
      <option name="INTERPRETER_OPTIONS" value="" />
      <option name="PARENT_ENVS" value="true" />
      <envs>
        <env name="PYTHONUNBUFFERED" value="1" />
      </envs>
      <option name="SDK_HOME" value="" />
      <option name="WORKING_DIRECTORY" value="$PROJECT_DIR$" />
      <option name="IS_MODULE_SDK" value="true" />
      <option name="ADD_CONTENT_ROOTS" value="true" />
      <option name="ADD_SOURCE_ROOTS" value="true" />
      <EXTENSION ID="PythonCoverageRunConfigurationExtension" runner="coverage.py" />
      <option name="SCRIPT_NAME" value="$PROJECT_DIR$/build.py" />
      <option name="PARAMETERS" value="pycharm_builder run_unit_tests run_integration_tests --environment=develop" />
      <option name="SHOW_COMMAND_LINE" value="false" />
      <option name="EMULATE_TERMINAL" value="false" />
      <option name="MODULE_MODE" value="false" />
      <option name="REDIRECT_INPUT" value="false" />
      <option name="INPUT_FILE" value="" />
      <method v="2" />
    </configuration>
    <configuration name="build (production)" type="PythonConfigurationType" factoryName="Python">
      <module name="{project_name}" />
      <option name="INTERPRETER_OPTIONS" value="" />
      <option name="PARENT_ENVS" value="true" />
      <envs>
        <env name="PYTHONUNBUFFERED" value="1" />
      </envs>
      <option name="SDK_HOME" value="" />
      <option name="WORKING_DIRECTORY" value="$PROJECT_DIR$" />
      <option name="IS_MODULE_SDK" value="true" />
      <option name="ADD_CONTENT_ROOTS" value="true" />
      <option name="ADD_SOURCE_ROOTS" value="true" />
      <EXTENSION ID="PythonCoverageRunConfigurationExtension" runner="coverage.py" />
      <option name="SCRIPT_NAME" value="$PROJECT_DIR$/build.py" />
      <option name="PARAMETERS" value="pycharm_builder publish --environment=production" />
      <option name="SHOW_COMMAND_LINE" value="false" />
      <option name="EMULATE_TERMINAL" value="false" />
      <option name="MODULE_MODE" value="false" />
      <option name="REDIRECT_INPUT" value="false" />
      <option name="INPUT_FILE" value="" />
      <method v="2" />
    </configuration>
    <configuration name="Unittests in {project_name}/tests" type="tests" factoryName="Unittests" nameIsGenerated="true">
      <module name="{project_name}" />
      <option name="INTERPRETER_OPTIONS" value="" />
      <option name="PARENT_ENVS" value="true" />
      <option name="SDK_HOME" value="" />
      <option name="WORKING_DIRECTORY" value="" />
      <option name="IS_MODULE_SDK" value="true" />
      <option name="ADD_CONTENT_ROOTS" value="true" />
      <option name="ADD_SOURCE_ROOTS" value="true" />
      <EXTENSION ID="PythonCoverageRunConfigurationExtension" runner="coverage.py" />
      <option name="_new_pattern" value="&quot;test_*.py&quot;" />
      <option name="_new_additionalArguments" value="&quot;&quot;" />
      <option name="_new_target" value="&quot;$PROJECT_DIR$/tests&quot;" />
      <option name="_new_targetType" value="&quot;PATH&quot;" />
      <method v="2" />
    </configuration>
    <list>
      <item itemvalue="Python.build (develop)" />
      <item itemvalue="Python.build (develop no-tests)" />
      <item itemvalue="Python.run tests (PyBuilder)" />
      <item itemvalue="Python.build (production)" />
      <item itemvalue="Python tests.Unittests in {project_name}/tests" />
    </list>
  </component>
</project>"""



#   -*- coding: utf-8 -*-
#   Copyright 2020 Diego Barrantes
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

from pathlib import Path
import xml.etree.ElementTree as ET

from pybuilder.core import init, task

import pybuilder_pycharm_workspace.constants as const
import pybuilder_pycharm_workspace.messages as msg
from pybuilder_pycharm_workspace.errors import MissingPropertyError, NoPyCharmConfigDirError
from pybuilder_pycharm_workspace.helpers import fill_and_write_template, underscore
from pybuilder_pycharm_workspace.resources import templates as templates


@init
def initialise_plugin(project):
	"""
	Sets project defaults.

	:param pybuilder.core.Project project: PyBuilder project instance
	:return: None
	"""
	const.IML_FILENAME = const.IML_FILENAME.format(project_name=project.name)
	project.set_property_if_unset('pycharm_workspace_main_version', '2019')


@task(description=msg.TASK_DESCRIPTION_GENERATE_PYCHARM_WORKSPACE)
def generate_pycharm_workspace(project, logger):
	"""
	Main plugin task.

	It creates the ``.idea`` directory for the project and tells PyCharm the interpreter it should use with the
	current project (``.\\venv`` in project's directory by default).

	:param pybuilder.core.Project project: PyBuilder project instance
	:param pybuilder.core.Logger logger: PyBuilder logger instance
	:return: None
	"""
	logger.info(msg.WORKSPACE_START)

	if not project.get_property('pycharm_workspace_project_path'):
		raise MissingPropertyError('pycharm_workspace_project_path')

	add_project_interpreter(project, logger)
	add_project_idea_directory(project, logger)
	logger.info(msg.WORKSPACE_FINISH)


def add_project_interpreter(project, logger):
	"""
	Function in charge of adding project's virtual environment to PyCharm config.

	It checks the ``jdk.table.xml`` file for the existence of a previously configured interpreter with the same name.
	If it doesn't exist, this function adds the interpreter for PyCharm for the virtual environment belonging to the
	project. In case it finds one, this will be replaced by the new one.

	:param pybuilder.core.Project project: PyBuilder project instance
	:param pybuilder.core.Logger logger: PyBuilder logger instance
	:return: None
	:raises NoPyCharmConfigDirError: If the plugin can't find PyCharm config directory
	"""
	logger.info(msg.INTERPRETER_START)

	for directory in [".PyCharm", ".PyCharmCE"]:
		pycharm_config_path = list(Path.home().glob(f"{directory}{project.get_property('pycharm_workspace_main_version')}*"))[-1:]
		if len(pycharm_config_path):
			break
	if not len(pycharm_config_path):
		raise NoPyCharmConfigDirError
	pycharm_config_path = pycharm_config_path[0]
	logger.debug(msg.INTERPRETER_PYCHARM_LATEST.format(pycharm_config_name=pycharm_config_path.name))
	pycharm_config_path = pycharm_config_path / 'config' / 'options'
	logger.debug(msg.INTERPRETER_PYCHARM_LATEST_PATH.format(pycharm_config_path=pycharm_config_path))

	project_interpreter = templates.INTERPRETER.format(
		project_name=project.get_property('pycharm_workspace_project_path').name,
		project_path=str(project.get_property('pycharm_workspace_project_path')).replace('\\\\', '\\'))
	project_interpreter = ET.fromstring(project_interpreter)
	interpreter_name = project_interpreter.find('name').attrib['value']
	logger.debug(msg.INTERPRETER_NEW_NAME.format(interpreter_name=interpreter_name))
	project.set_property('pycharm_workspace_project_interpreter_name', interpreter_name)

	interpreters_file_path = str(pycharm_config_path / 'jdk.table.xml')
	interpreters_file = ET.parse(interpreters_file_path)
	interpreters_table = interpreters_file.getroot().find(".//component[@name='ProjectJdkTable']")
	old_project_interpreter = interpreters_table.find(f".//name[@value='{interpreter_name}']..")
	if old_project_interpreter is not None:
		logger.info(msg.INTERPRETER_FOUND)
		interpreters_table.remove(old_project_interpreter)
	interpreters_table.append(project_interpreter)
	interpreters_file.write(interpreters_file_path)

	logger.debug(msg.INTERPRETER_FILE_OVERWRITTEN.format(interpreters_file_path=interpreters_file_path))


def add_project_idea_directory(project, logger):
	"""
	Function in charge of adding PyCharm's ``.idea`` directory for the project.

	This function should be always executed before launching ``add_project_interpreter``. It creates the directory and
	includes the minimum necessary files on it filling the templates included in the plugin with the information of
	the current project. These files are:

	* IML file with project name defining project structure (as source folder etc.)
	* ``modules.xml`` file with a reference to IML file
	* ``misc.xml`` file with a reference to the interpreter configured by ``add_project_interpreter`` function
	* ``workspace.xml`` file with run manager configurations to ease different PyBuilder buildings:

		* Build with development environment configuration
		* Build with development environment configuration and no testing
		* Only run tests
		* Build with production environment configuration

	:param pybuilder.core.Project project: PyBuilder project instance
	:param pybuilder.core.Logger logger: PyBuilder logger instance
	:return: None
	"""
	logger.info(msg.WORKSPACE_CREATING_IDEA_DIRECTORY)
	pycharm_idea_directory = project.get_property('pycharm_workspace_project_path') / '.idea'
	pycharm_idea_directory.mkdir(exist_ok=True)

	logger.debug(msg.WORKSPACE_CREATING_FILE.format(file_name=const.IML_FILENAME))
	unit_tests = templates.IML_SOURCEFOLDER_TEMPLATE.format(
		directory=project.get_property('dir_source_unittest_python')) if project.get_property('dir_source_unittest_python') else ''
	integration_tests = templates.IML_SOURCEFOLDER_TEMPLATE.format(
		directory=project.get_property('dir_source_integrationtest_python')) if project.get_property('dir_source_integrationtest_python') else ''
	fill_and_write_template(templates.IML_FILE, pycharm_idea_directory / const.IML_FILENAME,
	                        **{ 'source_dir': (Path('src') / underscore(project.name)).as_posix(),
                                'unit_tests': unit_tests,
	                            'integration_tests': integration_tests,
								'interpreter_name': project.get_property('pycharm_workspace_project_interpreter_name'),
	                            'output_directory': project.get_property('dir_target') })

	logger.debug(msg.WORKSPACE_CREATING_FILE.format(file_name=const.MODULES_FILENAME))
	fill_and_write_template(templates.MODULES_FILE, pycharm_idea_directory / const.MODULES_FILENAME,
	                        **{ 'project_file_name': const.IML_FILENAME })

	logger.debug(msg.WORKSPACE_CREATING_FILE.format(file_name=const.MISC_FILENAME))
	fill_and_write_template(templates.MISC_FILE, pycharm_idea_directory / const.MISC_FILENAME,
                            **{ 'project_interpreter_name': project.get_property('pycharm_workspace_project_interpreter_name') })

	logger.debug(msg.WORKSPACE_CREATING_FILE.format(file_name=const.WORKSPACE_FILENAME))
	fill_and_write_template(templates.WORKSPACE_FILE, pycharm_idea_directory / const.WORKSPACE_FILENAME,
	                        **{ 'project_name': project.name })



#   -*- coding: utf-8 -*-
#   Copyright 2020 Diego Barrantes
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

import pybuilder_pycharm_workspace.messages as msg


class MissingPropertyError(Exception):
	def __init__(self, property, message=None):
		"""
		This exception is raised when a property value is missing.

		:param str or None message: Custom exception message
		"""
		super().__init__(message if message else msg.MISSING_PROPERTY_ERROR.format(property=property))


class NoPyCharmConfigDirError(Exception):
	def __init__(self, message=None):
		"""
		This exception is raised when the plugin can't find PyCharm configuration directory.

		:param str or None message: Custom exception message
		"""
		super().__init__(message if message else msg.NO_PYCHARM_CONFIG_DIR_ERROR)


class WritingFileError(Exception):
	def __init__(self, file_path, directory, message=None):
		"""
		This exception is raised when the plugin is trying to write one of the configuration files to disk.

		:param str or None message: Custom exception message
		"""
		super().__init__(message if message else msg.WRITING_FILE_ERROR.format(file=file_path, directory=directory))



#   -*- coding: utf-8 -*-
#   Copyright 2020 Diego Barrantes
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

TASK_DESCRIPTION_GENERATE_PYCHARM_WORKSPACE = "Generates PyCharm workspace files fo the current project"

INTERPRETER_START = "Creating new Python interpreter for project's virtualenv"
INTERPRETER_PYCHARM_LATEST = "Latest PyCharm config directory detected: {pycharm_config_name}"
INTERPRETER_PYCHARM_LATEST_PATH = "PyCharm config directory detected: '{pycharm_config_path}'"
INTERPRETER_NEW_NAME = "New Python interpreter name: '{interpreter_name}'"
INTERPRETER_FOUND = "A Python interpreter found with the same name (replacing it)"
INTERPRETER_FILE_OVERWRITTEN = "'{interpreters_file_path}' file overwritten"

WORKSPACE_START = "Generating PyCharm project files"
WORKSPACE_CREATING_IDEA_DIRECTORY = "Creating new .idea directory"
WORKSPACE_CREATING_FILE = "Creating new {file_name} file in PyCharm .idea directory"
WORKSPACE_FINISH = "PyCharm workspace created"

MISSING_PROPERTY_ERROR = "Plugin property '{property}' not set in build.py file"
NO_PYCHARM_CONFIG_DIR_ERROR = "No PyCharm configuration directory found in user system path. Please launch PyCharm for the first time"
WRITING_FILE_ERROR = "There was an error trying to write '{file}' into {directory} directory"



#!/usr/bin/env python

from setuptools import setup
from setuptools.command.install import install as _install

class install(_install):
    def pre_install_script(self):
        pass

    def post_install_script(self):
        pass

    def run(self):
        self.pre_install_script()

        _install.run(self)

        self.post_install_script()

if __name__ == '__main__':
    setup(
        name = 'pybuilder-pycharm-workspace',
        version = '0.1.2',
        description = 'PyBuilder PyCharm Workspaces Plugin',
        long_description = 'External plugin for PyBuilder integration with PyCharm',
        author = 'Diego BM',
        author_email = 'diegobm92@gmail.com',
        license = 'Apache License, Version 2.0',
        url = 'https://github.com/yeuk0/pybuilder-pycharm-workspace',
        scripts = [],
        packages = [
            'pybuilder_pycharm_workspace',
            'pybuilder_pycharm_workspace.resources'
        ],
        namespace_packages = [],
        py_modules = [],
        classifiers = [
            'Development Status :: 3 - Alpha',
            'Programming Language :: Python'
        ],
        entry_points = {},
        data_files = [],
        package_data = {},
        install_requires = [],
        dependency_links = [],
        zip_safe = True,
        cmdclass = {'install': install},
        keywords = '',
        python_requires = '',
        obsoletes = [],
    )



#   -*- coding: utf-8 -*-
#   Copyright 2020 Diego Barrantes
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

import re

from pybuilder_pycharm_workspace.errors import WritingFileError


def fill_and_write_template(template, output_path, **fields):
	"""
	Fills a template with some values and writes the result in a specific location.

	:param str template: Text to format
	:param str output_path: Path where the new file will be saved
	:param dict[str, str] fields: Collection of template's fields with the new values to write
	:return: None
	:raises WritingFileError: If there was an error while trying to write the file
	"""
	try:
		project_metadata = template.strip().format(**fields)
		with open(output_path, 'w') as project_file:
			project_file.write(project_metadata)
	except:
		raise WritingFileError(template, output_path)


def underscore(word):
	"""
	``underscore()`` function taken from ``inflection`` library.

	More information:
		https://inflection.readthedocs.io/en/latest/#inflection.underscore

	:param str word: Text to process
	:return: Underscored word with no hyphens and lowercased.
	:rtype: str
	"""
	word = re.sub(r"([A-Z]+)([A-Z][a-z])", r"\1_\2", word)
	word = re.sub(r"([a-z\d])([A-Z])", r"\1_\2", word)
	word = word.replace('-', '_')
	return word.lower()
